#include <stdafx.h>
#include <time.h>
#include "DrawTransition.h"

DrawTransition::DrawTransition()
{
	m_nMaxSteps = 10;
	m_nCurrentStep = 0;
	m_bIsDrawingTransition = false;
	m_Effect = eRectOut;
}

void DrawTransition::StartRandom()
{
	int effect = int(m_Random.GenerateFloat() * eNumEffects);

	Start(eEffect(effect));
}

void DrawTransition::Start(eEffect theEffect)
{
	m_bIsDrawingTransition = true;
	m_nCurrentStep = 0;

	m_Effect = theEffect;
	switch(m_Effect)
	{
	case eWipeLeft:
	case eWipeRight:
		m_nMaxSteps = 15;
		break;
	case eWipeDown:
	case eWipeUp:
		m_nMaxSteps = 5;
		break;
	case eRectOut:
		m_nMaxSteps = 10;
		break;
	case eHorizontalBlindOut:
		m_nMaxSteps = 10;
		break;
	case eVerticalBlindOut:
		m_nMaxSteps = 5;
		break;
	case eMoveRight:
	case eMoveLeft:
		m_nMaxSteps = 15;
		break;
	default:
		break;
	}
}

void DrawTransition::FillEffect(ATL_DRAWINFO& di)
{
	RECT& rc = *(RECT*)di.prcBounds;

	int nWidth = rc.right - rc.left;
	int nHeight = rc.bottom - rc.top;
	int nWidth2 = nWidth / 2;
	int nHeight2 = nHeight / 2;
	int nMiddleH = rc.left + nWidth2;
	int nMiddleV = rc.top  + nHeight2;
	double fFrac = (double)m_nCurrentStep / m_nMaxSteps;

	switch(m_Effect)
	{
	case eWipeLeft:
		rc.left = rc.right - long(nWidth * fFrac);
		break;
	case eWipeRight:
		rc.right = rc.left + long(nWidth * fFrac);
		break;
	case eWipeDown:
		rc.bottom = long(rc.bottom * fFrac);
		break;
	case eWipeUp:
		rc.top = rc.bottom - long(nHeight * fFrac);
		break;
	case eRectOut:
		rc.left   = nMiddleH - long(nWidth2  * fFrac);
		rc.top    = nMiddleV - long(nHeight2 * fFrac);
		rc.right  = nMiddleH + long(nWidth2  * fFrac);
		rc.bottom = nMiddleV + long(nHeight2 * fFrac);
		break;
	case eHorizontalBlindOut:
		rc.left   = nMiddleH - long(nWidth2  * fFrac);
		rc.right  = nMiddleH + long(nWidth2  * fFrac);
		break;
	case eVerticalBlindOut:
		rc.top    = nMiddleV - long(nHeight2 * fFrac);
		rc.bottom = nMiddleV + long(nHeight2 * fFrac);
		break;
	}
		// Not sure if I like the rectangle look
//	::FillRect(di.hdcDraw, &rc, (HBRUSH)(COLOR_WINDOW + 1));	
	::FillRect(di.hdcDraw, &rc, (HBRUSH)(COLOR_BACKGROUND + 1));	
}

void DrawTransition::TranslateEffect(ATL_DRAWINFO& di)
{
	RECT& rc = *(RECT*)di.prcBounds;

	RECT& fillRc = rc;

	int nWidth  = rc.right  - rc.left;
	int nHeight = rc.bottom - rc.top;
	int nHDelta = nWidth / m_nMaxSteps;
	int nVDelta = nHeight / m_nMaxSteps;

	switch(m_Effect)
	{
	case eMoveRight:
		fillRc.right = rc.left + nWidth * m_nCurrentStep / m_nMaxSteps;
		::FillRect(di.hdcDraw, &fillRc, (HBRUSH)(COLOR_WINDOW + 1));

		::BitBlt(di.hdcDraw, rc.left + nHDelta, rc.top, nWidth - nHDelta, nHeight, 
				di.hdcDraw, rc.left, rc.top, SRCCOPY);
		break;
	case eMoveLeft:
		fillRc.left = rc.right - nWidth * m_nCurrentStep / m_nMaxSteps;
		::FillRect(di.hdcDraw, &fillRc, (HBRUSH)(COLOR_WINDOW + 1));

		::BitBlt(di.hdcDraw, rc.left, rc.top, nWidth, nHeight, 
				di.hdcDraw, rc.left + nHDelta, rc.top, SRCCOPY);
		break;
	}
}

void DrawTransition::Draw(ATL_DRAWINFO& di)
{
	switch(m_Effect)
	{
	case eWipeLeft:
	case eWipeRight:
	case eWipeDown:
	case eWipeUp:
	case eRectOut:
	case eHorizontalBlindOut:
	case eVerticalBlindOut:
		FillEffect(di);
		break;
	case eMoveRight:
	case eMoveLeft:
		TranslateEffect(di);
		break;
	}

	m_nCurrentStep++;
	if (m_nCurrentStep > m_nMaxSteps)
	{
		m_nCurrentStep = 0;
		m_bIsDrawingTransition = false;
	}
}

ATL_DRAWINFO* DrawTransition::BeginDraw(ATL_DRAWINFO& di)
{
#if 0 // work in progress
	HDC screenDc;
	screenDc = ::CreateDC("DISPLAY", NULL, NULL, NULL);
	::CreateCompatibleDC(screenDc, NULL));

	VERIFY(m_DragBitmap.CreateCompatibleBitmap(&screenDc, m_StartRect.Width(), m_StartRect.Height()));
	int nPlanes = m_MemDc.GetDeviceCaps(PLANES);
	int nBitsPixel = m_MemDc.GetDeviceCaps(BITSPIXEL);
	m_DragBitmap.CreateBitmap(m_StartRect.Width(), 
		m_StartRect.Height(), 
		nPlanes, nBitsPixel,
		NULL);

	m_pOldBitmap = m_MemDc.SelectObject(&m_DragBitmap);

	m_MemDc.OffsetViewportOrg(-m_StartRect.left, -m_StartRect.top);

	::DeleteDC(screenDc);

	return &m_DrawInfo;
#else
	return NULL;
#endif
}
