#include <stdafx.h>
#include "MyTime.h"

const TCHAR* k_strDays        = TEXT(" days ");
const TCHAR* k_strHours       = TEXT(" hours ");
const TCHAR* k_strMinutes     = TEXT(" minutes and ");
const TCHAR* k_strSeconds     = TEXT(" seconds");

const TCHAR* k_strColon = TEXT(":");


const int k_MilliSecondsInSecond = 100;
const int k_MilliSecondsInMinute = 60 * k_MilliSecondsInSecond;
const int k_MilliSecondsInHour   = 60 * k_MilliSecondsInMinute;
const int k_MilliSecondsInDay    = 24 * k_MilliSecondsInHour;


MyTime::MyTime()
{
	memset(&m_Time, 0, sizeof(SYSTEMTIME));
}

void MyTime::SetCurrentUTC()
{
	GetSystemTime(&m_Time);
}

void MyTime::AppendTwoDigit(TCHAR* str, int num, bool bInsertZero)
{
	if (num < 10)
	{
		if (bInsertZero)
		{
			_tcscat(str, TEXT("0"));
		}
		else
		{
			_tcscat(str, TEXT(" "));
		}
	}

	TCHAR buf[10] = {0};
	_itot(num, buf, 10);
	_tcscat(str, buf);	
}

int MyTime::GetMilliSecondTimeDifferenceFromNow()
{
	MyTime now;
	now.SetCurrentUTC();

	return GetMilliSecondTimeDifference(now);
}

int MyTime::GetMilliSecondTimeDifference(const MyTime& otherTime)
{
	// Convert SYSTEMTIME to FILETIME
	FILETIME ArrivalFileTime;
	SystemTimeToFileTime(&m_Time, &ArrivalFileTime);
	
	__int64 arrive;
	memcpy(&arrive, &ArrivalFileTime, sizeof(__int64));

	// Convert SYSTEMTIME to FILETIME
	FILETIME CurrentFileTime;
	SystemTimeToFileTime(&otherTime.GetTime(), &CurrentFileTime);
	
	// Copy FILETIME to LARGE_INTEGER
	__int64 today;
	memcpy(&today, &CurrentFileTime, sizeof(__int64));

	__int64 diff = today - arrive;

	return (int)(diff / 100000);
}

void MyTime::Set(int nYear, int nMonth, int nDay, int nHour, int nMinute, int nSecond)
{
	m_Time.wYear   = nYear;
	m_Time.wMonth  = nMonth;
	m_Time.wDay    = nDay; 
	m_Time.wHour   = nHour;
	m_Time.wMinute = nMinute;
	m_Time.wSecond = nSecond;
}

void MyTime::SplitMilliSecondTimeDifference(int diff)
{
	m_Time.wYear = 0;
	m_Time.wMonth = 0;
	m_Time.wDay  = (int) (diff / k_MilliSecondsInDay);
	diff -= m_Time.wDay * k_MilliSecondsInDay;

	m_Time.wHour = (int) (diff / k_MilliSecondsInHour);
	diff -= m_Time.wHour * k_MilliSecondsInHour;

	m_Time.wMinute = (int) (diff / k_MilliSecondsInMinute);
	diff -= m_Time.wMinute * k_MilliSecondsInMinute;

	m_Time.wSecond = (int) (diff / k_MilliSecondsInSecond);
}

void MyTime::AppendPeriodOfTime(TCHAR* work)
{
	AppendTwoDigit(work, GetDay(), false);

	_tcscat(work, k_strDays);

	AppendTwoDigit(work, GetHour(), false);

	_tcscat(work, k_strHours);

	AppendTwoDigit(work, GetMinute(), false);

	_tcscat(work, k_strMinutes);

	AppendTwoDigit(work, GetSecond(), false);

	_tcscat(work, k_strSeconds);
}

void MyTime::AppendTime(TCHAR* str)
{
	AppendTwoDigit(str, 
		(GetHour() < 13) ? (GetHour()) : (GetHour() - 12), false);
	
	_tcscat(str, k_strColon);

	AppendTwoDigit(str, GetMinute());

	_tcscat(str, k_strColon);

	AppendTwoDigit(str, GetSecond());

	_tcscat(str, TEXT(" "));

	if (GetHour() > 12)
	{
		_tcscat(str, TEXT("PM"));
	}
	else
	{
		_tcscat(str, TEXT("AM"));
	}
}

void MyTime::ChangeToTimeZone(MyTimeZone& tz)
{
	SystemTimeToTzSpecificLocalTime(
		(LPTIME_ZONE_INFORMATION)&tz.GetInfo(),  // Cast away const
		&m_Time, 
		&m_Time);
}

MyTimeZone::MyTimeZone()
{
	memset(&m_TzInfo, 0, sizeof(m_TzInfo));
}

void MyTimeZone::SetBrazilZone()
{
	m_TzInfo.Bias = 180; // 180min/60min = 3 hours

	m_TzInfo.StandardDate.wMonth = 2; // Feb
	m_TzInfo.DaylightDate.wDayOfWeek = 0; // Sunday
	m_TzInfo.StandardDate.wDay   = 2; // 2nd Sunday in October
	m_TzInfo.StandardDate.wHour  = 2; // 2 am
	
	m_TzInfo.DaylightDate.wMonth = 10;// Oct. 
	m_TzInfo.DaylightDate.wDayOfWeek = 0; // Sunday
	m_TzInfo.DaylightDate.wDay   = 3; // 3rd Sunday in October
	m_TzInfo.DaylightDate.wHour  = 2; // 2 am

	m_TzInfo.DaylightBias = -60; // Same as in most time zones
}

