// ScottsClockCtrl.cpp : Implementation of CScottsClockCtrl
#include "stdafx.h"
#include "InternetClock.h"
#include "ScottsClockCtrl.h"
#include "Routines/IFormat.h"

/////////////////////////////////////////////////////////////////////////////
// CScottsClockCtrl

const TCHAR* k_strBrazilTime  = TEXT("Time in Belo Horizonte, Brazil is ");
const TCHAR* k_strArriveTime  = TEXT("Scott has lived in Brazil for ");
const TCHAR* k_strStuffTime   = TEXT("My furniture will arrive in ");
const TCHAR* k_strBrazilPop   = TEXT("Population of Brazil is ");

CScottsClockCtrl::CScottsClockCtrl()
{
	m_bTimerOn = false;
	m_nSecCount = 0;
	m_nMilliSecCount = 0;
	m_eShowString = eShowFirst;
}

HRESULT CScottsClockCtrl::OnDraw(ATL_DRAWINFO& di)
{
	if (m_Transition.GetIsDrawingTransition())
	{
		m_Transition.Draw(di);
		return S_OK;
	}

	m_nSecCount++;
	if (m_nSecCount >= k_RotateEvery)
	{
		m_nSecCount = 0;

		m_eShowString++;
		if (m_eShowString >= eShowLast)
		{
			m_eShowString = eShowFirst; 
		}

		m_Transition.StartRandom();

		return S_OK;
	}

	DrawInfo(di);

	if (!m_bTimerOn)
	{
		TurnTimerOn();
	}

	return S_OK;
}

void CScottsClockCtrl::DrawInfo(ATL_DRAWINFO& di)
{
	RECT& rc = *(RECT*)di.prcBounds;

		// Not sure if I like the rectangle look
	::FillRect(di.hdcDraw, &rc, (HBRUSH)(COLOR_WINDOW + 1));

	TCHAR strBrazil[1024] = {0};

	switch (m_eShowString)
	{
	case eShowTimeInBrazil:
		CalcBrazilTimeString(strBrazil);
		break;
	case eShowTimeDifference:
		CalcTimeInBrazilString(strBrazil);
		break;
	case eShowTimeTillStuffArrives:
		CalcTimeTillStuffArrivesString(strBrazil);
		break;
	case eShowBrazilPopulation:
		CalcBrazilPopulation(strBrazil);
		break;
	default:
		break;
	}
  	COLORREF hOldColor = ::SetTextColor(di.hdcDraw, RGB(0, 0, 0));

	DrawText(di.hdcDraw, strBrazil, -1, &rc, DT_CENTER | DT_VCENTER | DT_SINGLELINE);

  	::SetTextColor(di.hdcDraw, hOldColor);
}

static CScottsClockCtrl* g_pCtrl = NULL;
static UINT g_TimerId = NULL;

CScottsClockCtrl::~CScottsClockCtrl()
{
	if (g_TimerId)
	{
		::KillTimer(NULL, g_TimerId);
		g_TimerId = NULL;
		g_pCtrl = NULL;
	}
	g_TimerId = NULL;
	g_pCtrl = NULL;
}


void CALLBACK TimerProc(HWND hwnd, UINT uMsg, UINT idEvent, DWORD dwTime)
{
	if (g_pCtrl)
	{
		BOOL bla;
		g_pCtrl->OnTimerEvent(0, 0, 0, bla);
	}
}

void CScottsClockCtrl::TurnTimerOn()
{
	if (m_bInPlaceActive)
	{
		g_pCtrl = this;
		g_TimerId = ::SetTimer(NULL, 0, k_nNumMilliseconds, TimerProc);
		m_bTimerOn = true;
	}
}

LRESULT CScottsClockCtrl::OnTimerEvent(
	UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	if (m_bInPlaceActive)
	{
		// Only redraw every second for most events
		m_nMilliSecCount++;
		if (m_nMilliSecCount > 1000 / k_nNumMilliseconds)
		{
			m_nMilliSecCount = 0;
		}

		if ((m_Transition.GetIsDrawingTransition() == false && 0 == m_nMilliSecCount) ||
			 m_Transition.GetIsDrawingTransition() == true)
		{
			// Active
			if (m_hWndCD != NULL)
				return ::InvalidateRect(m_hWndCD, NULL, FALSE); // Window based
			if (m_spInPlaceSite != NULL)
				return m_spInPlaceSite->InvalidateRect(NULL, FALSE); // Windowless
		}
	}

	return S_OK;
}

void CScottsClockCtrl::CalcBrazilTime(MyTime& localTime)
{
	localTime.SetCurrentUTC();

	MyTimeZone tmInfo;
	tmInfo.SetBrazilZone();

	localTime.ChangeToTimeZone(tmInfo);
}

void CScottsClockCtrl::CalcBrazilTimeString(TCHAR* str)
{
	MyTime LocalTime;
	CalcBrazilTime(LocalTime); 

	_tcscpy(str, k_strBrazilTime);

	LocalTime.AppendTime(str);
}

void CScottsClockCtrl::CalcTimeInBrazil(MyTime& timeDiff)
{
	MyTime BrazilArriveTimeUTC;

	BrazilArriveTimeUTC.Set(1998,
		8, // August
		28, // 28th
		8 + 3, // 8 am here -> 11 am in Greenwich
		15 // about 15 past
	);

	int diff = BrazilArriveTimeUTC.GetMilliSecondTimeDifferenceFromNow();

	timeDiff.SplitMilliSecondTimeDifference(diff);
}

void CScottsClockCtrl::CalcTimeInBrazilString(TCHAR* work)
{
	MyTime timeDiff;
	CalcTimeInBrazil(timeDiff);

	_tcscpy(work, k_strArriveTime);

	timeDiff.AppendPeriodOfTime(work);
}

void CScottsClockCtrl::CalcTimeTillStuffArrives(MyTime& timeDiff, bool& bHasArrived)
{
	MyTime BrazilArriveTimeUTC;

	BrazilArriveTimeUTC.Set(1998, 
		10, // Oct
		26, // 26th 
		8 + 3 // 8 am here -> 11 am in Greenwich
	);

	int diff = 0;
	if (BrazilArriveTimeUTC.GetMilliSecondTimeDifferenceFromNow() < 0)
	{
		bHasArrived = false;
		diff = -BrazilArriveTimeUTC.GetMilliSecondTimeDifferenceFromNow();
	}
	else
	{
		bHasArrived = true;
		diff = BrazilArriveTimeUTC.GetMilliSecondTimeDifferenceFromNow();
	}

	timeDiff.SplitMilliSecondTimeDifference(diff);
}

void CScottsClockCtrl::CalcTimeTillStuffArrivesString(TCHAR* work)
{
	MyTime timeDiff;
	bool bHasArrived;
	CalcTimeTillStuffArrives(timeDiff, bHasArrived);

	_tcscpy(work, k_strStuffTime);

	if (bHasArrived)
	{
		_tcscat(work, "-");
	}
	timeDiff.AppendPeriodOfTime(work);
}

// Brazilian population clock
// 162,255,794  (4:23 Sept. 23) + 3 = 17:23 
// 162.267,026  (2:32 Sept. 25)
//      11,232  (166,140 seconds) 
// source is http://www.ibge.gov.br/cgi-bin/popclock/pop2.exe?

void CScottsClockCtrl::CalcBrazilPopulation(TCHAR* work)
{
	_tcscpy(work, k_strBrazilPop);

	MyTime myTime;
	myTime.Set(1998, 9, 23, 17, 23);

	unsigned long secs = myTime.GetMilliSecondTimeDifferenceFromNow() / 100;

	unsigned long pop = 162255794;
	pop += secs * 11232L / 166140 ;
	
	IFormat fmt;
	_tcscat(work, fmt.Str(pop));
}


STDMETHODIMP CScottsClockCtrl::get_Message1(BSTR * pVal)
{
	return S_OK;
}

STDMETHODIMP CScottsClockCtrl::put_Message1(BSTR newVal)
{
	ATLTRACE(_T("PutMessage1\n"));
	SetMessage(0, newVal);
	Beep(300, 2000);
	return S_OK;
}


// Returns false if it succeeds
BOOL CScottsClockCtrl::SetMessage(int nIndex, const CComBSTR & str)
{
	if (nIndex < k_nMaxMessages)
	{
		m_Messages[nIndex] = str;
		return FALSE; // success
	}

	return TRUE; // Failed
}
